"""
This file is part of Apricopt.

Apricopt is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Apricopt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Apricopt.  If not, see <http://www.gnu.org/licenses/>.

Copyright (C) 2020-2022 Marco Esposito, Leonardo Picchiami.
"""

from typing import Dict, List
#from pyfmi import FMUModel

from apricopt.model.ModelInstance import ModelInstance

from abc import ABC, abstractmethod


class PyFMIModelInstance(ModelInstance):

    def __init__(self, model_obj):
        super().__init__(model_obj)
        self.time_step = -1
        self.horizon = -1

    def __delete__(self, instance):
        model_obj = self.model_obj
        del model_obj

    def set_parameter(self, param_name: str, value: float) -> None:
        model_obj = self.model_obj
        model_obj.set(param_name, value)

    @abstractmethod
    def set_parameters(self, params_values: Dict[str, float]) -> None:
        pass

    @abstractmethod
    def get_parameters_initial_values(self) -> Dict[str, float]:
        pass

    def get_compartment_initial_volumes(self) -> Dict[str, float]:
        return dict()

    @abstractmethod
    def get_system_input_variables(self) -> List[str]:
        pass

    @abstractmethod
    def get_system_output_variables(self) -> List[str]:
        pass

    def set_simulation_configuration(self, abs_tol: float, rel_tol: float, step_size: float) -> None:
        self.time_step = step_size
        default_time_step = self.model_obj.get_default_experiment_step()
        if step_size != default_time_step:
            print(f"The time_step={step_size} is different from the default_time_step={default_time_step}", flush=True)

    def set_simulation_duration(self, duration: float) -> None:
        self.horizon = duration
        default_horizon = self.model_obj.get_default_experiment_stop_time()
        if self.horizon != default_horizon:
            print(f"The horizon={self.horizon} is different from the default_horizon={default_horizon}", flush=True)

    def set_simulation_duration_with_step_size(self, horizon, time_step):
        self.set_simulation_duration(horizon)
        self.time_step = time_step
        default_time_step = self.model_obj.get_default_experiment_step()
        if time_step != default_time_step:
            print(f"The time_step={time_step} is different from the default_time_step={default_time_step}", flush=True)