"""
This file is part of Apricopt.

Apricopt is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Apricopt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Apricopt.  If not, see <http://www.gnu.org/licenses/>.

Copyright (C) 2020-2022 Sapienza University of Rome, Marco Esposito, Leonardo Picchiami.
"""

from typing import Type

from apricopt.model.Model import Model
from apricopt.model.ModelInstance import ModelInstance
from pyfmi import FMUModelCS2

from apricopt.simulation.fmu.pyfmi.PyFMIEngineCS import PyFMIEngineCS
from apricopt.simulation.fmu.pyfmi.fmi2.PyFMIModelInstanceFMI2 import PyFMIModelInstanceFMI2


class PyFMIEngineCS2(PyFMIEngineCS):
    def __init__(self):
        super().__init__()
        self.end_trajectory = False

    def load_model(self, model_filename: str, **exec_info) -> ModelInstance:
        model_obj: FMUModelCS2 = FMUModelCS2(model_filename)
        if 'end_trajectory' in exec_info:
            self.end_trajectory = exec_info['end_trajectory']

        return PyFMIModelInstanceFMI2(model_obj)

    def model_instance_class(self) -> Type[ModelInstance]:
        return PyFMIModelInstanceFMI2

    def start_fmu_simulation_config_building(self, model: Model, horizon: float) -> None:
        model_obj: PyFMIModelInstanceFMI2 = model.instance
        model_obj.start_fmu_initialisation(horizon)

    def end_fmu_simulation_config_building(self, model: Model) -> None:
        model_obj: PyFMIModelInstanceFMI2 = model.instance
        model_obj.end_fmu_initialisation()
