"""
This file is part of Apricopt.

Apricopt is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Apricopt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Apricopt.  If not, see <http://www.gnu.org/licenses/>.

Copyright (C) 2020-2021 Marco Esposito, Leonardo Picchiami.
"""

from typing import Dict, Union, List
from pyfmi import FMUModelCS2, FMUModelME2

from apricopt.simulation.fmu.pyfmi.PyFMIModelInstance import PyFMIModelInstance


class PyFMIModelInstanceFMI2(PyFMIModelInstance):

    def __init__(self, model_obj: Union[FMUModelCS2, FMUModelME2]):
        super().__init__(model_obj)

    def __delete__(self, instance):
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        del model_obj

    def get_system_input_variables(self) -> List[str]:
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        params = model_obj.get_input_list()
        return list(params.keys())

    def get_system_output_variables(self) -> List[str]:
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        params = model_obj.get_output_list()
        return list(params.keys())

    def start_fmu_initialisation(self, horizon: float) -> None:
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        model_obj.setup_experiment(start_time=0, stop_time=horizon)
        model_obj.enter_initialization_mode()

    def end_fmu_initialisation(self) -> None:
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        model_obj.exit_initialization_mode()

    def set_parameters(self, params_values: Dict[str, float]) -> None:
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        for param_id, value in params_values.items():
            model_obj.set(param_id, value)


    def get_parameters_initial_values(self) -> Dict[str, float]:
        values: Dict[str, float] = dict()
        model_obj: Union[FMUModelCS2, FMUModelME2] = self.model_obj
        params = self.get_system_input_variables()
        for param_id in params:
            values[param_id] = model_obj.get(param_id)
        return values